
/* I2C module */


/*
 * MASTER writes to a SLAVE
 *
 *  1) Send a START sequence
 *
 *  2) Send the I2C address of the SLAVE with the R/W bit low (even address)
 *
 *  3) Send the device register you want to write to
 *
 *  4) Send the data byte
 *
 *  5) [Optionally, send any further data bytes]
 *
 *  6) Send the STOP sequence.
 *
 *
 * MASTER reads from a SLAVE
 *
 *  1) Send a START sequence
 *
 *  2) Send I2C address of the SLAVE with the R/W bit low (even address)
 *
 *  3) Send device register you want to read from
 *
 *  4) Send a START sequence again (a repeated start)
 *
 *  5) Send I2C address of the SLAVE with the R/W bit high (odd address)
 *
 *  6) Read data byte from SLAVE
 *
 *  7) Send the STOP sequence.
 */


/*
 * SRF08
 * Current - 15mA Typ. 3mA Standby.
 * Frequency - 40KHz
 * Range - 3cm - 6m.
 * Max Analogue Gain - Variable 94 to 1025 in 32 steps.
 * Connection - Standard I2C Bus.
 * Light Sensor - Front Facing light sensor.
 */


#ifdef _PIC18
  #include  <pic18.h>
#else
  #include <pic.h>
#endif

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "Loki_2620.h"
#include "LokiI2C.h"
#include "delay.h"
#include "serial.h"

#define LED5	RA2

/* error codes for I2C routines */
#define WORKING 0
#define DONE 1
#define I2C_ERROR -1

/* states for I2C_tasks */
#define WAIT_TASK			0
#define SONAR_TASK 			1
#define TASK_S2				2
#define TASK_S3				3
#define LIGHT_TASK 			4
#define COMPASS_TASK		5
#define COMPASS_CALIB_TASK	6
#define RANGE_SETUP_TASK	7
#define GAIN_SETUP_TASK		8
#define NEW_ADDRESS_TASK	9
#define NEW_ADDRESS_TASK1	10
#define NEW_ADDRESS_TASK2	11
#define NEW_ADDRESS_TASK3	12
#define TPA81_READ_TASK		13
#define LAST_TASK	 		99

/* sequence code for new address */
#define SEQUENCE_1         0xA0
#define SEQUENCE_2         0xAA
#define SEQUENCE_3         0xA5


/* data for I2C_Tasks */
signed   int  SRF08_Range = 999;
unsigned char SRF08_Light;
unsigned char SRF08_Ver = 99;
unsigned char SRF08GainSelect = 2;
unsigned char CMPS03_Ver = 99;
unsigned int  CMPS03_Heading = 999;
unsigned char I2C_ReadBuff[10];
unsigned char srf08_addr = SRF08_000;
unsigned int  srf08_data[6];
unsigned char tpa81_data[10];
unsigned char srf08_LightData[6];
unsigned char srf08_chan = 0;
unsigned char NewSRF08_Address;



const unsigned char const GainTable[] = {GAIN_30CM, GAIN_47CM, GAIN_103CM};
const unsigned char const RangeTable[] = {MAX_RANGE_30CM, MAX_RANGE_47CM, MAX_RANGE_103CM};
const unsigned char const SRF08_Chan[] = {SRF08_000, SRF08_060, SRF08_120, SRF08_180, SRF08_240, SRF08_300};

extern void PutString(const char *ptr);


/*----------------------------------------------------------------------------------*/
void I2C_Tasks(void)

/*
 * Routine to select I2C tasks with priority
 *
 * Waits for SET_RANGE, SET_COMPASS_READ, SET_LIGHT, etc, task flag set,
 *
 * compass and range tasks are set by explicit range request or timed event
 *
 * 06/01/08 alm
 */

 	{
	static unsigned char Task = WAIT_TASK;
	char RetVal;
	char Cmd[40];

	switch(Task)
		{
		case WAIT_TASK:
			{
			if(Event2.Event2 != 0)		/* if any I2C tasks set */
				{
				if(CHK_RANGE_READ)			/* Start I2C SONAR task? */
					Task = SONAR_TASK;
				else
					if(CHK_COMPASS_READ)			/* Start I2C COMPASS task? */
						Task = COMPASS_TASK;
					else
						if(CHK_LIGHT_READ)			/* Start I2C LIGHT task? */
							Task = LIGHT_TASK;
						else
							if(CHK_GAIN_SETUP)
								Task = GAIN_SETUP_TASK;
							else
								if(CHK_RANGE_SETUP)
									Task = RANGE_SETUP_TASK;
								else
									if(CHK_COMPASS_CALIB)
										Task = COMPASS_CALIB_TASK;
									else
										if(CHK_NEW_ADDRESS)
											Task = NEW_ADDRESS_TASK;
									else
										if(CHK_TPA81_READ)
											Task = TPA81_READ_TASK;

//						Sonar Version?
//						Compass Version?
				}// Event2
			}
		break;

		case SONAR_TASK:	/* request range (ping)  */
			{
			if((RetVal = SendI2C_Cmd(srf08_addr, COMMAND_REG, DIST_IN_INCH)) == DONE)
				{
				Task = TASK_S2;
				SystemTick2 = 0;
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					{
					Task = WAIT_TASK;
					CLR_RANGE_READ;		/* quit I2C RANGE task */
					}
			}
		break;

		case TASK_S2:
			{
			if(SystemTick2 >= MS150_TICK)	/* 150 mS delay for ping return (60 * 2500 uS) */
				Task = TASK_S3;
			}
		break;

		case TASK_S3:	/* get range (word) light (byte) and version (byte) */
			{
			if((RetVal = ReadI2C_Data(srf08_addr, SRF08_VERSION, 4, I2C_ReadBuff)) == DONE)
				{
				SRF08_Ver = I2C_ReadBuff[0];  /* start with SRF08_VERSION */
				SRF08_Light = I2C_ReadBuff[1];
				SRF08_Range = I2C_ReadBuff[3] + (I2C_ReadBuff[2] << 8);

				/* put range and light into table for "group" readback */
				srf08_data[srf08_chan] = SRF08_Range;
				srf08_LightData[srf08_chan] = SRF08_Light;

				Task = WAIT_TASK;

				SET_RANGE_RPT;	/* request a report of the sonar range */

				/*
				 * maybe make a "report" command?
				 *
				 * probably want commands to always report, but use report cmd to
				 * enable reports from "running" 'bot.
				 */
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					Task = WAIT_TASK;

			if(srf08_addr == SRF08_300)
//			if(srf08_chan > 5)
				{
//			SET_RANGE_RPT;	/* request a report of the sonar range */
				CLR_RANGE_READ;		/* I2C RANGE cycle done */
				}

//			srf08_addr += 0x02;
			}
		break;

		case LIGHT_TASK:		/* get light */
			{
			if((RetVal = ReadI2C_Data(srf08_addr, SRF08_LIGHT,1, I2C_ReadBuff)) == DONE)
				{
				SRF08_Light = I2C_ReadBuff[0];
				Task = WAIT_TASK;
				SET_LIGHT_RPT;
//			PutString("light\r\n");

				CLR_LIGHT_READ;		/* I2C LIGHT task done */
				}
			}
		break;

		case COMPASS_TASK:		/* get compass heading (word) & version (byte) */
			{
			if((RetVal = ReadI2C_Data(CMPS03_ADDR, CMPS03_REV, 4, I2C_ReadBuff)) == DONE)
				{
				CMPS03_Ver = I2C_ReadBuff[0];
				CMPS03_Heading = I2C_ReadBuff[3] + (I2C_ReadBuff[2] << 8);

				SET_COMPASS_RPT;
				Task = WAIT_TASK;
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					Task = WAIT_TASK;

			CLR_COMPASS_READ;	/* I2C COMPASS task done */
			}
		break;

		case COMPASS_CALIB_TASK:	/* calibrate compass */
			{
			if(RetVal = SendI2C_Cmd(CMPS03_ADDR, CMPS03_CALIB, 0xFF) == DONE)
				{
				Task = WAIT_TASK;
			PutString("Fix (Calib) CMPS03 Compass\r\n");
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					Task = WAIT_TASK;

			CLR_COMPASS_CALIB;	/* I2C COMPASS CALIB task done */
			}
		break;

		case GAIN_SETUP_TASK:
			{
			if((RetVal = SendI2C_Cmd(srf08_addr, GAIN_REG, GainTable[SRF08GainSelect])) == DONE)
				{
		sprintf(Cmd, "set SRF08 gain %d\r\n", GainTable[SRF08GainSelect]);
		PutString(Cmd);

				Task = RANGE_SETUP_TASK;	/* now do gain */
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					Task = WAIT_TASK;

			CLR_GAIN_SETUP;	/* I2C gain setup task done */
			}
		break;

		case RANGE_SETUP_TASK:
			{
			if((RetVal = SendI2C_Cmd(srf08_addr, RANGE_REG, RangeTable[SRF08GainSelect])) == DONE)
				{
				Task = WAIT_TASK;
		sprintf(Cmd, "set SRF08 range %d\r\n", RangeTable[SRF08GainSelect]);
		PutString(Cmd);
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					Task = WAIT_TASK;

			CLR_RANGE_SETUP;	/* I2C range setup task done */
			}
		break;

		case NEW_ADDRESS_TASK:	/* has to send three sequence bytes, then new address... */
			{
//reset			srf08_addr = 0xE2;

			if((RetVal = SendI2C_Cmd(srf08_addr, COMMAND_REG, SEQUENCE_1) == DONE))
				{
				Task = NEW_ADDRESS_TASK1;
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					Task = WAIT_TASK;
			}
		break;

		case NEW_ADDRESS_TASK1:
			{
			if((RetVal = SendI2C_Cmd(srf08_addr, COMMAND_REG, SEQUENCE_2) == DONE))
				{
				Task = NEW_ADDRESS_TASK2;
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					Task = WAIT_TASK;
			}
		break;

		case NEW_ADDRESS_TASK2:
			{
			if((RetVal = SendI2C_Cmd(srf08_addr, COMMAND_REG, SEQUENCE_3) == DONE))
				{
				Task = NEW_ADDRESS_TASK3;
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					Task = WAIT_TASK;
			}
		break;

		case NEW_ADDRESS_TASK3:
			{
			if((RetVal = SendI2C_Cmd(srf08_addr, COMMAND_REG, NewSRF08_Address) == DONE))
				{
				Task = WAIT_TASK;
		sprintf(Cmd, "Set Addr: Old %02X, New %02X set\r\n", srf08_addr, NewSRF08_Address);
		PutString(Cmd);
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					Task = WAIT_TASK;

			CLR_NEW_ADDRESS;	/* I2C new address task done */
			}
		break;

		case TPA81_READ_TASK:
			{
		sprintf(Cmd, "TPA81 addr %0X, reg %02X\r\n", TPA81_ADDR, TPA81_SERVO_CMD);
		PutString(Cmd);

			if((RetVal = ReadI2C_Data(TPA81_ADDR, TPA81_AMBIENT, 9, I2C_ReadBuff)) == DONE)
//			if((RetVal = ReadI2C_Data(TPA81_ADDR, TPA81_REV, 1, I2C_ReadBuff)) == DONE)
//			if((RetVal = SendI2C_Cmd(TPA81_ADDR, TPA81_SERVO_CMD, 5) == DONE))
				{
		PutString("TPA81 Read OK\r\n");

				SET_TPA81_RPT;
				Task = WAIT_TASK;
				}
			else
				if(RetVal == (char)(I2C_ERROR))
					{
					Task = WAIT_TASK;
				
					sprintf(Cmd, "I2C Error %d\r\n", RetVal);
					PutString(Cmd);
					}
				else
					if(RetVal == (char)(WORKING))
						{
						Task = WAIT_TASK;
					
						sprintf(Cmd, "I2C WORKING %d\r\n", RetVal);
						PutString(Cmd);
						}

			CLR_TPA81_READ;	/* I2C task done */

			}
		break;

		default:
			{
			Task = WAIT_TASK;
			}
		break;
		}

//	if(RetVal == I2C_ERROR)	/* quit the task if an error */
//		Task = WAIT_TASK;
 	}

/*----------------------------------------------------------------------------------*/
char SendI2C_Cmd(unsigned char I2Caddr, unsigned char I2Creg, unsigned char I2Cdata)

/*
 * function to send I2C command
 * NO BLOCKING
 *
 * 6/1/08 alm
 *
 * look and report SSPOV, WCOL?
 *
 * errata:  don't set RCEN unless the following bits are clear:
 *		RCEN ACKEN, RCEN, PEN, RSEN and SEN.   A3, A4 silicon
 *
 * errata: Clock Rate w/BRG is wrong.  Use 59H instead of 99 (63h) A3, A4 silicon
 */


	{
	char RetVal;
	static unsigned char RangeState;
//	char Cmd[40];

	RetVal = WORKING;

	switch(RangeState)
		{
		case RANGE_IDLE:			/* 0 */
			{
			RangeState = RANGE_START;
			}
		break;

		case RANGE_START:			/* 1 */
			{
			BCLIF  = 0;				/* clear Bus Collision Interrupt Flag bit */
			WCOL   = 0;				/* clear Write Collision Detect Flag */
			SEN    = 1;				/* Initiate Start */
			SystemTick2 = 0;
			RangeState = RANGE_START_OK;
			}
		break;

		case RANGE_START_OK:		/* 2 i2cStart */
			{
			if(!BCLIF && !WCOL)		/* if no collision */
				{
				RangeState = RANGE_SEN;
				SystemTick2 = 0;
				}
			else
				if(SystemTick2 >= MS10_TICK)	/* 10 mS timeout  */
					{
			PutString("Collision1\r\n");
					RangeState = RANGE_STOP2; /* STOLEN!  */
					}
			}
		break;

		case RANGE_SEN:
			{
			if(!SEN)				/* 3 wait for Initiate Start condition clear */
				{ 					/* i2cWrite(addr) */
				ACKDT = 0;			/* want acknowledge */
				SSPIF  = 0;			/* Clear interrupt flag */
				SSPBUF = I2Caddr;	/* I2C address */
				RangeState = RANGE_ADDR;
				}
			else
				if(SystemTick2 >= MS100_TICK)	/* 100 mS timeout */
					{
			PutString("***Timeout1***\r\n");
					RangeState = RANGE_STOP2; /* STOLEN!  */
					}

//	sprintf(Cmd, "RS1 state %02d\r\n", RangeState);
//	PutString(Cmd);
			}
		break;

		case RANGE_ADDR:
			{
			if(SSPIF)				/* 4 wait for interrupt flag */
				{					/* i2cWrite(reg) */
				SSPIF  = 0;			/* Clear interrupt flag */
				SSPBUF = I2Creg;	/* I2C register */
				RangeState = RANGE_WR_WAIT;
				}
			}
		break;

		case RANGE_WR_WAIT:
			{
			if(SSPIF)				/* 5 wait for interrupt flag */
				{					/* i2cWrite(data) */
				SSPIF  = 0;			/* Clear interrupt flag */
				SSPBUF = I2Cdata;	/* data to send */
				RangeState = RANGE_STOP;
				}
			}
		break;

		case RANGE_STOP:
			{
			if(SSPIF)				/* 6 wait for interrupt flag */
				{					/* i2cStop */
				SSPIF = 0;			/* Clear interrupt flag */
				PEN   = 1;			/* Stop Condition Enable bit */

				RangeState = RANGE_STOP_WAIT;
				}
			}
		break;

		case RANGE_STOP_WAIT:
			{
			if(!PEN);				/* 7 wait for Stop Condition clear */
				{
				RetVal = DONE;
				RangeState = RANGE_IDLE;
				}
			}
		break;

		case RANGE_STOP2:	/* 23 STOLEN! if we need to recover from timeout? */
			{
//LED5 = 1;  /* 0 off, 1 on */

			I2C_Init();		/* re-initialize I2C */

		SystemTick2 = 0;

		RangeState = RANGE_RESET2;
			}
		break;

		case RANGE_RESET2:  /* STOLEN!  */
			{
			if(SystemTick2 >= MS1000_TICK)	/* 1000 mS timeout (400 * 2500 uS) */
				{
				RetVal = I2C_ERROR;	/* work out an error code return */
				RangeState = RANGE_IDLE;
//LED5 = 0;  /* 0 off, 1 on */

//	sprintf(Cmd, "***Reset*** %ld\r\n", pass);
//	PutString(Cmd);
				}
			}
		break;


		default:
			{
//			sprintf(Cmd, "C CRASH %d\r\n", RangeState);
//			PutString(Cmd);

			RetVal = I2C_ERROR;	/* work out an error code return */
			RangeState = RANGE_IDLE;
			}
		break;
		}

	return(RetVal);
	}

/*--------------------------------------------------------------------------*/
char ReadI2C_Data(unsigned char I2Caddr, unsigned char I2Creg,
	unsigned char BytesToRead, char *I2C_Buff)

/*
//void RangeReadFSM(void)
 * function to read I2C data word
 * NO BLOCKING
 *
 * 2/1/08 alm
 *
 * FIX:
 * modify to take a pointer to data returned and a data count instead
 * STOP assigning data read directly to parameters
 *
 * errata:  don't set RCEN unless the following bits are clear:
 *		RCEN ACKEN, RCEN, PEN, RSEN and SEN.   A3, A4 silicon
 *
 * errata: Clock Rate w/BRG is wrong.  Use 59H instead of 99 (63h) A3, A4 silicon
 *
 * NOTE!!!  I2C_Buff pointer passed is not used, I2C_ReadBuff is used directly !!!!!!  FIX  !!!!
 */

	{
	char RetVal;
	static unsigned char RangeState2;
	static unsigned char Count;
//	char Cmd[40];

//	static unsigned char RangeState2 = RANGE_START2;
//	static long pass = 0;

	RetVal = WORKING;


	switch(RangeState2)
		{
		case RANGE_START2:			/* 9 i2cStart */
			{
			BCLIF  = 0;				/* clear Bus Collision Interrupt Flag bit */
			WCOL = 0;				/* clear Write Collision Detect Flag */
			SEN = 1;				/* Initiate Start */
			SystemTick2 = 0;
			Count = 0;
			RangeState2 = RANGE_START_OK2;
			}
		break;

		case RANGE_START_OK2:		/* 10  */
			{
			if(!BCLIF && !WCOL)		/* if no collision */
				{
				SystemTick2 = 0;
				RangeState2 = RANGE_SEN2;
				}
			else
				if(SystemTick2 >= MS10_TICK)	/* 10 mS timeout  */
					{
	PutString("Collision2\r\n");
					RangeState2 = RANGE_STOP2;
					}

//	sprintf(Cmd, "RS2 state %02d\r\n", RangeState);
//	PutString(Cmd);
			}
		break;

		case RANGE_SEN2:
			{
			if(!SEN)				/* 11 wait for Initiate Start condition clear */
				{					/* i2cWrite(addr) */
				ACKDT  = 0;			/* Want acknowledge */
				SSPIF  = 0;			/* Clear interrupt flag */
				SSPBUF = I2Caddr;	/* I2C address */
				RangeState2 = RANGE_ADDR2_WAIT;
				}
			else
				if(SystemTick2 >= MS100_TICK)	/* 100 mS timeout  */
					{
					RangeState2 = RANGE_STOP2;
	PutString("***Timeout2***\r\n");
//	sprintf(Cmd, "***Timeout2*** %ld\r\n", pass);
//	PutString(Cmd);
					}

//	sprintf(Cmd, "RS2 state %02d\r\n", RangeState2);
//	PutString(Cmd);
			}
		break;

		case RANGE_ADDR2_WAIT:
			{
//	sprintf(Cmd, "RS2 state2 %02d\r\n", RangeState2);
//	PutString(Cmd);
			if(SSPIF)				/* 12 wait for interrupt flag */
				{					/* i2cWrite(reg) */
				SSPIF  = 0;			/* Clear interrupt flag */
				SSPBUF = I2Creg;	/* I2C register */
				RangeState2 = RANGE_REG2_WAIT;
				}
			}
		break;

		case RANGE_REG2_WAIT:
			{
			if(SSPIF)				/* 13 wait for interrupt flag */
				{					/* i2cRestart */
				SSPIF = 0;			/* Clear interrupt flag */
				RSEN  = 1;			/* set repeat start bit */
				RangeState2 = RANGE_RESTART_WAIT;
				}
			}
		break;

		case RANGE_RESTART_WAIT:	/* restart so that we can give it a read address */
			{
			if(!RSEN)				/* 14 wait for restart */
				{
				SSPIF  = 0;			/* Clear interrupt flag */
				SSPBUF = I2Caddr + 1;	/* i2cWrite(addr + 1) */
				RangeState2 = RANGE_ADDR_RD_WAIT;
				}
			}
		break;

		case RANGE_ADDR_RD_WAIT:
			{
			if(SSPIF)				/* 15 wait for interrupt flag */
				{					/* i2cRead with ACK	*/
				SSPIF = 0;			/* Clear interrupt flag */
				RCEN  = 1;			/* Initiate read sequence */
				RangeState2 = RANGE_WAIT_BF;
				}
			}
		break;

		case RANGE_WAIT_BF:
			{
			if(BF)					/* 16 wait for high byte of range */
				{
				I2C_ReadBuff[Count++] = SSPBUF;	/* data byte */

				/* use I2C_Buff ???? */

									/* BF is cleared by reading SSPBUF */
				/* loop for data */
				if(Count >= BytesToRead)
					{
					ACKDT = 1;			/* indicate we're done by NO ack on last byte */
					RangeState2 = RANGE_ACK_WAIT2;
					}
				else
					RangeState2 = RANGE_ACK_WAIT;

				ACKEN = 1;			/* Start acknowledge sequence */
				}
			}
		break;

		case RANGE_ACK_WAIT:
			{
			if(!ACKEN)				/* 17 Wait until acknowledge complete */
				{					/* i2cRead with ACK */
//  if(ACKSTAT != TRUE ?? )
//		report no acknowlegde?? */

				RCEN = 1;			/* Initiate read sequence */
				RangeState2 = RANGE_WAIT_BF;	/* loop back for next byte */
				}
			}
		break;


		case RANGE_ACK_WAIT2:
			{
			if(!ACKEN)				/* 19 Wait until acknowledge complete */
				{					/* i2cStop */
				PEN = 1;			/* Stop Condition Enable bit */
				RangeState2 = RANGE_WAIT_END;
				}
			}
		break;

		case RANGE_WAIT_END:
			{
			if(!PEN)				/* 20 wait for Stop Condition clear */
				{
//not here!	CLR_RANGE_READ;
				RangeState2 = RANGE_START2;
				RetVal = DONE;
				}
			}
		break;

		case RANGE_STOP2:	/* 23 if we need to recover from timeout? */
			{
//LED5 = 1;  /* 0 off, 1 on */
			I2C_Init();		/* re-initialize I2C */

		SystemTick2 = 0;
		RangeState2 = RANGE_RESET2;
			}
		break;

		case RANGE_RESET2:	/* 24 */
			{
			if(SystemTick2 >= MS1000_TICK)	/* 1000 mS timeout (400 * 2500 uS) */
				{
				RetVal = I2C_ERROR;	/* work out an error code return */
				RangeState2 = RANGE_START2;
//LED5 = 0;  /* 0 off, 1 on */
//	sprintf(Cmd, "***Reset*** %ld\r\n", pass);
//	PutString(Cmd);
				}
			}
		break;

		default:
			{
//			sprintf(Cmd, "R CRASH %d\r\n", RangeState2);
//			PutString(Cmd);

			RetVal = I2C_ERROR;		/* work out an error code return */
			RangeState2 = RANGE_START2;
			}
		break;
		}

	return(RetVal);
	}

/*----------------------------------------------------------------------------------*/

