
/*
 * parser files
 *
 * C - Compass read
 * D - A/D read
 * F - Fix Compass
 * G - Sonar Gain
 * I - IR range
 * S - Sonar range
 * R - Reset
 */

#ifdef _PIC18
  #include  <pic18.h>
#else
  #include <pic.h>
#endif

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "Loki_2620.h"
#include "PIChead.h"
#include "serial.h"
#include "LokiI2C.h"

/* parse messages */

#define ERROR 	  0x0000
#define VERSION   0x0001
#define QUERY1    0x0002
#define QUERY2    0x0004
#define QUERY3    0x0008
#define SERVO_CMD 0x0010
#define TIME	  0x0020
#define SPEED     0x0040
#define SEQUENCE  0x0088
#define RST	 	  0x0100
#define OFFSET	  0x0200
#define BEHAVIOR  0x0408
#define U_RANGE	  0x0808

#define VIEW	  0x1044
#define ADDR	  0x1084
#define EXEC	  0x1104
#define SAVE	  0x1144 // 1024
#define EXEC2	  0x1184 // 1144
#define EXEC3	  0x1284
#define CALIB	  0x2000
#define SERVO_POS 0x0011
#define REV		  0x1001

#define START_STEP 2


/* function prototypes */

unsigned int GetParm(char **str);
unsigned char CompressStr(char **str);
void ParseCmds(char *CmdStr);
char *GetCanned(void);

extern void PutString(const char *ptr);
extern void PutChar(char ch);
extern char *GetString(void);

extern unsigned char SRF08GainSelect;

/*-------------------------------------------------------------------------*/
void ParseCmds(char *CmdStr)

/*
 * Function to parse out the commands from a received
 * or canned string
 */
	{
	unsigned int Retval, SequenceParm1, SequenceParm2;
	unsigned char len;
//	char Cmd[55];

	do
		{
		if(CmdStr == NULL)		/* end of command string */
			break;

		len = strlen(CmdStr);		/* new/compressed length of command */

//		sprintf(Cmd, "len %d\r\n", len);	/* these debug messages make len = 0 not to be detected */
//		PutString(Cmd);

		if(len == 0)	/* no characters in string */
			{
//			LED5_OFF;  /* off error LED */
			break;
			}

		/* parse the string for command tokens and parameters */
		Retval = ParseBlock(&CmdStr, &SequenceParm1, &SequenceParm2);

//		sprintf(Cmd, "retval %02X\r\n", retval);
//		PutString(Cmd);


		switch(Retval & 0x3F)
			{
			case VERSION:			/* "VER" command */
				{
				if(Retval == REV)
					{
					PutString(SixStr);
					}
				else
					{
//					PutString("SSC32-1.06XE\r");
					PutString("SSC32-ALM\r");
					}

//				LED5_OFF;  /* off error LED */
				}
			break;

			case QUERY3:			/* "QPLn" returns player level (4 bytes) */
				{
				if(Retval == U_RANGE)	/* ranging request */
					{
					/* already started in parse block: */
					/* SONAR (I2C) request */
					/* Fix Compass (Calib) */
					/* Read Compass */
					/* Read Light */
					/* Read pyro data */

					/* SRF08 Sonar range/gain setting needs parameters */
					 if(CHK_GAIN_SETUP)
						{
						if(SequenceParm1 < 3)
						 	SRF08GainSelect = SequenceParm1;

//				sprintf(Cmd, "Parm1 %d, Parm2 %d\r\n", SequenceParm1, SequenceParm2);
//				PutString(Cmd);
						}

					if(CHK_TPA81_READ)
						{
//							PutString("TPA81 Read\r\n");
						}

					/* set new address for SRF08 needs parameters */
					 if(CHK_NEW_ADDRESS)
						{
//				sprintf(Cmd, "Parm1 %d, Parm2 %d\r\n", SequenceParm1, SequenceParm2);
//				PutString(Cmd);

						if(SequenceParm1 <= 5)	/* OLD six allowed addresses */
					 		srf08_addr = SRF08_Chan[SequenceParm1];
						else
							srf08_addr = SRF08_Chan[0];	/* default to E0 */

						if(SequenceParm2 <= 5)	/* NEW six allowed addresses */
					 		NewSRF08_Address = SRF08_Chan[SequenceParm2];
						else
							NewSRF08_Address = SRF08_Chan[0];	/* default to E0 */

//					sprintf(Cmd, "Sel old addr %02X, new addr %02X\r\n", srf08_addr, NewSRF08_Address);
//					PutString(Cmd);
						}
				    }
				}
			break;

			default:				/* what was that? */
				{
				if(Retval == RST)
					{
//					LED5_OFF;	/* off */
					PutString("RESET\r\n");
					}
				else
					{
					*CmdStr = 0;
					PutString("ERROR\r\n");
//					LED5_ON;  /* on */
					}
				}
			break;
			}
		}
	while(*CmdStr != 0 && len != 0);
	}

/*------------------------------------------------------------------------*/

unsigned ParseBlock(char **cmd, unsigned int *Sequence1, unsigned int *Sequence2)

/*
 * Parse routine
 *
 * SSC32-1.06XE<CR>
 *
 * VER  Get version #
 * REV	Get rev #
 * ESC  Clear error
 * S    Range request (Sonar)
 * I    Range request (IR)
 * C	Compass
 * F	Fix compass
 * D	A/D readings
 * A	new SRF08 Address
 * L	Light reading
 * G	Set SRF08 Gain
 */

	{
	unsigned int retval;
	char *str, chr1;
//	char Cmd[55];

	*Sequence1 = 0;
	*Sequence2 = 0;
	str = *cmd;
	retval = ERROR;

	if(stricmp(str, "VER") == 0)	/* respond to SIM query */
		{
		*cmd = str + 3;			/* update parse position */
		return(VERSION);
		}
	else
	  if(strcmp(str, "REV") == 0)
	  	{
		*cmd = str + 3;			/* update parse position */
		return(REV);
		}
	  else
		if(stricmp(str, "ESC") == 0)		/* ESC cmd resets */
			{
			*cmd = str + 3;		/* update parse position */
			return(RST);
			}
		else
			{
			chr1 = *str++;		/* get at next char in string */


			/* no 'B'. 'J', 'H', 'N' yet actually more...  */

			if((chr1 != '#') && (chr1 != 'Q') && (chr1 != 'S') && (chr1 != 'C')
				&& (chr1 != 'K') && (chr1 != 'R') && (chr1 != 'U') && (chr1 != 'X')
				&& (chr1 != 'Y') && (chr1 != 'V') && (chr1 != 'E') && (chr1 != 'A')
				&& (chr1 != 'I') && (chr1 != 'T') && (chr1 != 'Z') && (chr1 != 'W')
				&& (chr1 != 'O') && (chr1 != 'F') && (chr1 != 'M') && (chr1 != 'G')
				&& (chr1 != 'L') && (chr1 != 'D') && (chr1 != 'S'))
				return(ERROR);
			}

	if(chr1 == 'A')
		{
		*Sequence1 = GetParm(&str);		/* get current address */
		*Sequence2 = GetParm(&str);		/* get new address */
		*cmd = str;
		SET_NEW_ADDRESS;
		return(U_RANGE);
		}

	if(chr1 == 'T')
		{
		*cmd = str;
		SET_TPA81_READ;
//		SET_TPA81_RPT;
		return(U_RANGE);
		}

	if(chr1 == 'I')				/* IR Range */
		{
		*cmd = str;				/* update parse position */
		SET_IR_RPT;				/* new IR report */

		return(U_RANGE);
		}

	if(chr1 == 'D')				/* A/D report */
		{
		*cmd = str;				/* update parse position */
		SET_AD_RPT;				/* report 8 A/D reads */
		return(U_RANGE);
		}

	if(chr1 == 'F')				/* Fix Compass (Calib) */
		{
		*cmd = str;
		SET_COMPASS_CALIB;
		return(U_RANGE);
		}

	if(chr1 == 'L')				/* Read Light */
		{
		*cmd = str;
		SET_LIGHT_READ;
		return(U_RANGE);
		}

	if(chr1 == 'G')				/* set SRF08 gain range */
		{
		*Sequence1 = GetParm(&str);		/* get gain choice */
		*cmd = str;
		SET_GAIN_SETUP;
		return(U_RANGE);
		}

	if(chr1 == 'C')				/* Read Compass */
		{
		*cmd = str;
		SET_COMPASS_READ;
		return(U_RANGE);
		}

	if(chr1 == 'S')				/* Sonar Range */
		{
		*cmd = str;				/* update parse position */
		SET_RANGE_READ;		/* SONAR (I2C) request (and report) */
		return(U_RANGE);
		}

	if(chr1 == 'R')				/* RESET */
		{
		*cmd = str;				/* update parse position */
		return(BEHAVIOR);		/* stop at end of sequence */
		}

	*cmd = str;					/* update parse position */
	return(retval);
	}

/*------------------------------------------------------------------------*/
unsigned char CompressStr(char **str)

/*
 * routine to compress a string by
 * removing whitespace and ignoring
 * control codes
 *
 * updates string, returns length of string
 *
 * 1/24/08 alm
 *
 * add delimiter ','
 * 12/21/10 alm
 */

	{
	char chr;
	unsigned char i, j, len;
	char *str2;

	str2 = *str;
	len = strlen(str2);
	i = 0;

	if(str2 != NULL)
		for(j = 0; j <= len; j++)
			{
			chr = toupper(str2[j]);

			/* allowed characters */
			if((chr >= '0' && chr <= '9') || (chr >= 'A' && chr <= 'Z') || chr == '#' || chr == ','
				|| chr == ESC || chr == '+' || chr == '-')
				str2[i++] = chr;
			}

	str2[i] = 0;		/* terminate string */
	*str = str2;		/* update command string pointer */

	return(i);
	}

/*------------------------------------------------------------------------*/
unsigned int GetParm(char **str)

/*
 * routine to get the numeric value of a parameter
 *
 * "uses" only ASCII numbers, stops on first non-number
 * updates passed command pointer
 *
 * returns parameter (integer)
 *
 * might want to allow it to handle +/- signs?
 *
 * 01/21/08 alm
 *
 * add removal of trailing delimiter ','
 * 12/21/10 alm
 */

	{
	unsigned char j, len;
	unsigned int parm;
	char chrx;
	char *str2;
//	char Cmd[44];

	str2 = *str;
	len = strlen(str2);
	parm = 0;

// sprintf(Cmd, "len %d str %s\r\n", len, str2);
// PutString(Cmd);

	/* assemble characters into a number */
	for(j = 0; j < len; j++)
		{
		chrx = *str2;	/* look at a character */

		if(chrx >= '0' && chrx <= '9')	/* allow numbers only */
			{
			parm *= 10;
			parm += (chrx - '0');
			}
		else
			break;	/* don't eat the character */

		*str2++;	/* advance to next character */
		}

	/*  take out delimiter ',' */
	len = strlen(str2);

	for(j = 0; j < len; j++)
		{
		chrx = *str2;	/* look at a character */

		if(chrx == ',')	/* eat the delimiter */
			*str2++;
		else
			break;		/* don't eat the character */
		}

	*str = str2;	/* update command string pointer */
	return(parm);
	}

/*------------------------------------------------------------------------*/
