/*
 *  Serial routines
 */


#ifdef _PIC18
  #include  <pic18.h>
#else
  #include <pic.h>
#endif

#include <ctype.h>
#include <stdio.h>

#include "Loki_2620.h"
#include "serial.h"

/* ----LOCAL VARIABLES---- */

char RxBuff[RX_BUFF_SIZE];			/* Input Buffer */
volatile char RxHeadPtr;

char TxBuff[TX_BUFF_SIZE];			/* Output Buffer */
volatile char TxHeadPtr, TxTailPtr;

void PutData(const char *ptr, unsigned char len);


#ifdef WANT_ADDRESS_STUFF
	char ch;
	char OutputBuffer[TX_BUFF_SIZE];	/* Output buffer for HexLib routines */
#endif

/*-------------------------------------------------------------------------*/
void RxCharDevice(void)

/*
 *	Receive character Interrupt routine.
 */

	{
    char InChar;


    if(RCIF && RCIE)			/* in isr too */
    	{						/* Character received */
        InChar = RCREG; 		/* Get char which clears interrupt flag */

    	if(OERR || FERR)
    		{					/* Overrun error */
    		CREN = 0;			/* Disable to clear overrun */
    		CREN = 1;			/* then enable again */
			}					/* So new interrupts come through */

        if(InChar == 0x0A)		/* Ignore linefeeds */
        	return;

    	if(!CHK_RX_MSG_FLAG)
    		{
			if(InChar == 0x0D)	/* CR signifies end of string to receive */
				{
				RxBuff[RxHeadPtr] = '\0';
				RxHeadPtr = 0;	/* Point to start of buffer again */
				SET_RX_MSG_FLAG; /* Signal message available */
				}
			else
				{
				RxBuff[RxHeadPtr++] = InChar;

				if(CHK_ECHO_FLAG)
					 PutChar(InChar);

				if(RxHeadPtr == RX_BUFF_SIZE) /* Prevent overflows of buffer */
					 --RxHeadPtr;
				}
    		} // chk RX MSG FLG
	   	}
	}


/*-------------------------------------------------------------------------*/
void TxCharDevice(void)

/*
 *	Transmit Character Interrupt Routine.
 */

	{

	if(TXIF && TXIE)	/* checked in isr too */
		{
		TXIF = 0;										/* clear the interrupt flag */

		if(TxHeadPtr != TxTailPtr)						/* if buffer not empty */
			{
			TXREG = TxBuff[TxHeadPtr];					/* send the char */
			TxHeadPtr = ++TxHeadPtr & TX_BUFF_MASK;		/* bump and wrap the pointer */
			}
		else
			TXIE = 0;	/* if buffer is empty, disable the TX interrupt */
		}
	}

/*-------------------------------------------------------------------------*/
#ifdef TX_INTERRUPT_ENABLED

/*
 *	routine to send a char to serial
 *
 * sets up XMIT interrupts if desired
 */

	void PutSerial(unsigned char chr)
		{
		static unsigned char ndx;

		// Wait while buffer is full as interrupt changes Front pointer.
		ndx = (TxTailPtr + 1) & TX_BUFF_MASK;

		// One char is removed by interrupt routine while loop exits.

		while(ndx == TxHeadPtr);	/* BLOCKS if no room in BUFFER!!  */
//	PEIE = 0;			/* don't let interrupt modify what we look at */

		if(TXIE == 0)		/* we will enable TX interrupt on first char to send */
			{
			TXREG = chr;	/* send the first char now */
			TXIE = 1;		/* ENABLE TX interrupt to handle sending chars */
			}
		else
			{
			TxBuff[TxTailPtr++] = chr;	/* char into XMIT buffer */
			TxTailPtr &= TX_BUFF_MASK;	/* wrap the buffer */
			}

//	PEIE = 1;			/* Let interrupt finish */
		}
#else

/*
 * routine to just wait for xmit to be empty...BLOCKS!
 */

	void PutSerial(unsigned char chr)
		{
		while(!TXIF);		/* BLOCKS! */

		TXREG = chr;
		}
#endif


/*-------------------------------------------------------------------------*/
void InitSerial(unsigned char baud)

/*
 *     Initialize the Serial Port and all Queues and Buffers.
 */

	{	// Sets baud rate etc.
	baud = (25000/1156);
	SPBRG = baud;
    RxHeadPtr = 0;

	TxHeadPtr = 0;
	TxTailPtr = 0;

	TXIE = 0;			/* Tx ints disabled initially */
	RCIE = 0;			/* Rx interrupts disabled initially */

	TXSTA = TXINIT;		/* sets up TXEN=1, BRGH=0 */
	RCSTA = RXINIT;		/* sets up SPEN=1 CREN=1 */

	TXEN = 1;			/* enable USART transmitter. */

//	SPEN = 1;	 		/* +++ Enable serial port. */

	RCIP = 0;			/* assign low priority to USART interrupts (should be already) */
	TXIP = 0;

	RCIF = 0;			/* clear recieve flag and enable receive interrupts */
	RCIE = 1;

	TXIF = 0;			/* clear xmit flag, enable interrupts LATER */
	TXIE = 0;
	}

/*-------------------------------------------------------------------------*/
void PutChar(char chr)

/*
 * function to send a char to comm port
 */

	{
	PutSerial(chr);
//	_usart_putc(chr);
	}

/*-------------------------------------------------------------------------*/
void PutData(const char *ptr, unsigned char len)

/*
 * function to send a specified number of bytes from a string
 * 12/22/10 alm
 */

{
int i;


	for(i = 0; i < len; i++)
		PutChar(*ptr++);
}

/*-------------------------------------------------------------------------*/
void PutString(const char *ptr)

/*
 * function to send a string to comm port
 */

	{
	 unsigned char chr;

	while(chr = *ptr)
		{
		PutChar(chr);
		ptr++;
		}
	}

/*-------------------------------------------------------------------------*/
unsigned char GetChar(void)

/*
 * function to get a char from comm port
 */

	{

	if(CHK_RX_MSG_FLAG)			/* if buffer has message */
		{
		if(RxBuff[RxHeadPtr] == '\0')	/* if buffer empty */
			{				 	/* Return null and prepare for next line. */
			CLR_RX_MSG_FLAG;  	/* Allow interrupt routine to process chars again. */
			RxHeadPtr = 0;
			return('\0');
			}
		else  			/* Otherwise return the next character in the input stream. */
			return(RxBuff[RxHeadPtr++]);
		}
	else
		return('\0');
	}

/*-------------------------------------------------------------------------*/
char* GetString(void)

/*
 * function to get a string from comm port
 */

	{
	CLR_RX_MSG_FLAG;  	/* Allow interrupt routine to process chars again. */

	return(&RxBuff[RxHeadPtr]);
	}

/*-------------------------------------------------------------------------*/
void PutCRLF(void)
/*
 * function to send a newline to comm port
 * old DOS functionality?
 */
	{
	PutChar('\r');
	PutChar('\n');
	}

/*-------------------------------------------------------------------------*/

